import dataclasses

from sohstationviewer.model.mseed_data.record_reader_helper import Unpacker
from sohstationviewer.model.reftek.reftek_data.packet import \
    eh_et_payload_end_in_packet
from sohstationviewer.model.reftek.reftek_data.header import PacketHeader


def read_eh_et_packet(packet: bytes, unpacker: Unpacker):
    """
    Process an EH/ET packet and get its extended header and required part of
    the payload.
    :param packet: the bytes that make up the given EH/ET packet.
    :param unpacker: the unpacker to use to decode the data.
    :return: the extended header and truncated payload of the given EH/ET
        packet.
    """
    event_number = int(packet[16:18].hex())
    data_stream_number = int(packet[18:19].hex())
    flags = unpacker.unpack('B', packet[22:23])[0]
    data_format = packet[23:24].hex().upper()

    extended_header = EHETExtendedHeader(event_number, data_stream_number,
                                         flags, data_format)
    # The largest possible data point has a size of 4  bytes, so we need to
    # grab at least data.
    payload = packet[24:eh_et_payload_end_in_packet]
    return extended_header, payload


@dataclasses.dataclass
class EHETExtendedHeader:
    """
    A collection of some useful information about an EH/ET packet. Technically,
    EH/ET packets do not have extended headers. We name this class what it is
    due to the way obspy.Reftek130 (and consequently, core.Reftek130) stores
    the data of processed packets. For more information, refer to
    Reftek130._data.
    """
    event_number: int
    data_stream_number: int
    flags: int
    data_format: str

    def __post_init__(self):
        self.channel_number = 0
        self.number_of_samples = 0


@dataclasses.dataclass
class EHETPacket:
    """
    The decoded data of an EH/ET packet. The extended_header field is to ensure
    compatibility with dt_packet.DTPacket. EH/ET packets do not have an
    extended header otherwise.
    """
    header: PacketHeader
    extended_header: EHETExtendedHeader
    data: bytes
