import dataclasses

from obspy.io.reftek.util import bcd

from sohstationviewer.model.mseed_data.record_reader_helper import Unpacker
from sohstationviewer.model.reftek.reftek_data.header import PacketHeader

import numpy


@dataclasses.dataclass
class SOHExtendedHeader:
    """
    A collection of dummy data for some information needed so that
    core.Reftek130 can understand SOH packets.

    core.Reftek130 focuses on reading waveform data, so it wants information
    available in the waveform packets (EH/ET/DT). However, core.Reftek130 also
    supports SOH packets, which does not contain the required information. As
    a result, we need to store dummy data in its place.
    """
    event_number: int
    data_stream_number: int
    channel_number: int
    number_of_samples: int
    flags: int
    data_format: str


@dataclasses.dataclass
class SOHPacket:
    """
    The decoded data of an SOH packet. The extended_header field is to ensure
    compatibility with dt_packet.DTPacket. SOH packets do not have an
    extended header otherwise.
    """
    header: PacketHeader
    extended_header: SOHExtendedHeader
    data: bytes


def bcd_16bit_int(_i):
    """
    Reimplement a private function of the same name in obspy. Kept here in case
    the private function is removed in future obspy version.
    :param _i: the byte string to convert into a 16-bite integer
    :return: a 16-bit integer
    """
    _i = bcd(_i)
    return _i[0] * 100 + _i[1]


def read_soh_packet(packet: bytes, unpacker: Unpacker):
    """
    Process an SOH packet and get its extended header and poyload.
    :param packet: the bytes that make up the given SOH packet.
    :param unpacker: the unpacker to use to decode the data.
    :return: the extended header and payload of the given SOH packet.
    """

    event_number = bcd_16bit_int(numpy.frombuffer(packet[16:18], numpy.uint8))
    data_stream_number = bcd(numpy.frombuffer(packet[18:19], numpy.uint8))
    channel_number = bcd(numpy.frombuffer(packet[19:20], numpy.uint8))
    number_of_samples = bcd_16bit_int(numpy.frombuffer(packet[20:22], numpy.uint8))
    flags = unpacker.unpack('B', packet[22:23])[0]
    data_format = packet[23:24].hex().upper()

    extended_header = SOHExtendedHeader(event_number, data_stream_number,
                                        channel_number, number_of_samples,
                                        flags, data_format)
    payload = packet[24:]
    return extended_header, payload
